CREATE OR REPLACE PACKAGE BODY lista_enteros_pkg AS

/* Funcion para comprobar la existencia de la infraestructura de listas
*/
FUNCTION existe_lista_listas RETURN BOOLEAN IS
	tabla USER_TABLES.TABLE_NAME%TYPE;
BEGIN
	SELECT TABLE_NAME INTO tabla FROM USER_TABLES WHERE TABLE_NAME = 'INDICE_LISTAS';
	RETURN TRUE;
EXCEPTION
	WHEN no_data_found THEN
		RETURN FALSE;
	WHEN OTHERS THEN
		RAISE;
END;

/* Procedimiento para crear la lista de listas */
PROCEDURE crea_lista_listas IS
	cid INTEGER;
	result INTEGER;
	username VARCHAR2(100);
BEGIN
	SELECT USER INTO username FROM DUAL;
	cid := DBMS_SQL.OPEN_CURSOR ();
	DBMS_SQL.PARSE (cid, 'CREATE TABLE ' || username || '.indice_listas (nombre VARCHAR2(100) PRIMARY KEY)',
		DBMS_SQL.V7);
	result := DBMS_SQL.EXECUTE (cid);
	DBMS_SQL.PARSE (cid, 'CREATE TABLE ' || username || '.lista_listas (nombre VARCHAR2(100) REFERENCES ' || 
		username || '.indice_listas, posicion NUMBER(3), valor NUMBER(5), PRIMARY KEY (nombre, posicion))',
		DBMS_SQL.V7);
	result := DBMS_SQL.EXECUTE (cid);
	DBMS_SQL.CLOSE_CURSOR (cid);
	RETURN;
END;

/* Procedimiento para crear una lista de enteros
   Parametros:
      - nombre: cadena de caracteres que contiene el nombre de la lista que se crea (I)
*/
PROCEDURE crear_lista (nombre IN VARCHAR2) IS
BEGIN
	INSERT INTO indice_listas VALUES (nombre);
EXCEPTION
	WHEN OTHERS THEN
		IF (SQLCODE() = -1) THEN
			RAISE_APPLICATION_ERROR (-20001, 'Lista ' || nombre || ' ya existe');
		ELSE
			RAISE;
		END IF;
END;

/* Procedimiento para destruir una lista enteros
   Parametros:
      - nombre: cadena de caracteres que contiene el nombre de la lista que se destruye (I)
*/
PROCEDURE destruir_lista (nombreABorrar IN VARCHAR2) IS
BEGIN
	DELETE FROM indice_listas WHERE NOMBRE = nombreABorrar;
	IF (SQL%NOTFOUND) THEN
		RAISE_APPLICATION_ERROR (-20000, 'Lista ' || nombreABorrar || ' no existe');
	END IF;
END;

/* Procedimiento para insertar un elemento entero en una posicion dada de una lista de enteros
   Parametros:
      - nombre: cadena de caracteres que contiene el nombre de la lista en la que se inserta (I)
      - elemento: elemento entero que se inserta (I)
      - posicion: posicion entera de la lista en la que se inserta (I)
*/
PROCEDURE insertar_en_lista (nombre IN VARCHAR2, elemento IN INTEGER, posicion IN INTEGER) IS
BEGIN
	INSERT INTO LISTA_LISTAS VALUES (nombre, posicion, elemento);
EXCEPTION
	WHEN OTHERS THEN
		IF (SQLCODE () = -2291) THEN
			RAISE_APPLICATION_ERROR (-20000, 'Lista ' || nombre || ' no existe');
		ELSIF (SQLCODE () = -1) THEN
			RAISE_APPLICATION_ERROR (-20050, 'Elemento ' || to_char(posicion) || ' de lista ' || 
				nombre || ' ya existe');
		ELSE
			RAISE;
		END IF;
END;

/* Funcion para extraer un elemento entero de una posicion dada de una lista de enteros
   Parametros:
      - nombre: cadena de caracteres que contiene el nombre de la lista en la que se inserta (I)
      - elemento: elemento entero que se inserta (I)
      - posicion: posicion entera de la lista en la que se inserta (O)
*/
FUNCTION extraer_de_lista (nombreABorrar IN VARCHAR2, pos IN INTEGER) RETURN INTEGER IS
	nodo LISTA_LISTAS%ROWTYPE;
	nom INDICE_LISTAS.NOMBRE%TYPE;
BEGIN
	BEGIN
		SELECT * INTO nodo FROM LISTA_LISTAS WHERE NOMBRE = nombreABorrar AND POSICION = pos;
	EXCEPTION
		WHEN no_data_found THEN
			BEGIN
				SELECT NOMBRE INTO nom FROM INDICE_LISTAS WHERE NOMBRE = nombreABorrar;
			EXCEPTION
				WHEN no_data_found THEN
					RAISE_APPLICATION_ERROR (-20000, 'Lista ' || nombreABorrar || ' no existe');
				WHEN OTHERS THEN
					RAISE;
			END;
			RAISE_APPLICATION_ERROR (-20051, 'Elemento ' || to_char(pos) || ' de lista ' || 
					nombreABorrar || ' no existe');
		WHEN OTHERS THEN
			RAISE;
	END;
	DELETE FROM LISTA_LISTAS WHERE NOMBRE = nombreABorrar AND POSICION = pos;
	RETURN (nodo.valor);
END;

END;
/


